@@ -560,7 +560,6 @@ static int
       return STAT_NORETRY;
    }
 
-   xfd->addr  = &xioaddr_openssl_listen;
    xfd->dtype = XIODATA_OPENSSL;
 
    while (true) {	/* loop over failed attempts */
@@ -930,7 +929,6 @@ int
 
    //*ipproto = IPPROTO_TCP;
 
-   xfd->addr  = &xioaddr_openssl;
    xfd->dtype = XIODATA_OPENSSL;
 
    retropt_bool(opts, OPT_OPENSSL_FIPS, &opt_fips);
@@ -1777,15 +1775,17 @@ static int openssl_handle_peer_certificate(struct single *xfd,
 #if WITH_IP6
 			case 16: /* IPv6 */
 			   inet_ntop(AF_INET6, data, aBuffer, sizeof(aBuffer));
-			   xioip6_pton(peername, &ip6bin);
-			   if (memcmp(data, &ip6bin, sizeof(ip6bin)) == 0) {
-			      Debug2("subjectAltName \"%s\" matches peername \"%s\"",
-				    aBuffer, peername);
-			      ok = 1;
-			   } else {
-			      Info2("subjectAltName \"%s\" does not match peername \"%s\"",
-				    aBuffer, peername);
-			   }			      
+			   if (peername != NULL) {
+			      xioip6_pton(peername, &ip6bin);
+			      if (memcmp(data, &ip6bin, sizeof(ip6bin)) == 0) {
+			         Debug2("subjectAltName \"%s\" matches peername \"%s\"",
+					aBuffer, peername);
+			         ok = 1;
+			      } else {
+			         Info2("subjectAltName \"%s\" does not match peername \"%s\"",
+				       aBuffer, peername);
+			      }
+			   }
 			   break;
 #endif
 			}
